@@ -28,6 +28,7 @@ from __future__ import absolute_import
 
 import zlib
 
+from autobahn.util import public
 from autobahn.websocket.compress_base import PerMessageCompressOffer, \
     PerMessageCompressOfferAccept, \
     PerMessageCompressResponse, \
@@ -51,22 +52,23 @@ class PerMessageDeflateMixin(object):
 
     EXTENSION_NAME = "permessage-deflate"
     """
-   Name of this WebSocket extension.
-   """
+    Name of this WebSocket extension.
+    """
 
     WINDOW_SIZE_PERMISSIBLE_VALUES = [8, 9, 10, 11, 12, 13, 14, 15]
     """
-   Permissible value for window size parameter.
-   Higher values use more memory, but produce smaller output. The default is 15.
-   """
+    Permissible value for window size parameter.
+    Higher values use more memory, but produce smaller output. The default is 15.
+    """
 
     MEM_LEVEL_PERMISSIBLE_VALUES = [1, 2, 3, 4, 5, 6, 7, 8, 9]
     """
-   Permissible value for memory level parameter.
-   Higher values use more memory, but are faster and produce smaller output. The default is 8.
-   """
+    Permissible value for memory level parameter.
+    Higher values use more memory, but are faster and produce smaller output. The default is 8.
+    """
 
 
+@public
 class PerMessageDeflateOffer(PerMessageCompressOffer, PerMessageDeflateMixin):
     """
     Set of extension parameters for `permessage-deflate` WebSocket extension
@@ -81,15 +83,16 @@ class PerMessageDeflateOffer(PerMessageCompressOffer, PerMessageDeflateMixin):
         :param params: Output from :func:`autobahn.websocket.WebSocketProtocol._parseExtensionsHeader`.
         :type params: list
 
-        :returns: object -- A new instance of :class:`autobahn.compress.PerMessageDeflateOffer`.
+        :returns: A new instance of :class:`autobahn.compress.PerMessageDeflateOffer`.
+        :rtype: obj
         """
 
         # extension parameter defaults
-        acceptMaxWindowBits = False
-        acceptNoContextTakeover = True
-        # acceptNoContextTakeover = False # FIXME: this may change in draft
-        requestMaxWindowBits = 0
-        requestNoContextTakeover = False
+        accept_max_window_bits = False
+        accept_no_context_takeover = True
+        # accept_no_context_takeover = False # FIXME: this may change in draft
+        request_max_window_bits = 0
+        request_no_context_takeover = False
 
         # verify/parse client ("client-to-server direction") parameters of permessage-deflate offer
         for p in params:
@@ -100,7 +103,7 @@ class PerMessageDeflateOffer(PerMessageCompressOffer, PerMessageDeflateMixin):
             val = params[p][0]
 
             if p == 'client_max_window_bits':
-                ##
+                #
                 # see: https://tools.ietf.org/html/draft-ietf-hybi-permessage-compression-18
                 # 8.1.2.2. client_max_window_bits
 
@@ -118,17 +121,17 @@ class PerMessageDeflateOffer(PerMessageCompressOffer, PerMessageDeflateMixin):
                             raise Exception("illegal extension parameter value '%s' for parameter '%s' of extension '%s'" % (val, p, cls.EXTENSION_NAME))
                         else:
                             # FIXME (maybe): possibly forward/process the client hint!
-                            # acceptMaxWindowBits = val
-                            acceptMaxWindowBits = True
+                            # accept_max_window_bits = val
+                            accept_max_window_bits = True
                 else:
-                    acceptMaxWindowBits = True
+                    accept_max_window_bits = True
 
             elif p == 'client_no_context_takeover':
                 # noinspection PySimplifyBooleanCheck
                 if val is not True:
                     raise Exception("illegal extension parameter value '%s' for parameter '%s' of extension '%s'" % (val, p, cls.EXTENSION_NAME))
                 else:
-                    acceptNoContextTakeover = True
+                    accept_no_context_takeover = True
 
             elif p == 'server_max_window_bits':
                 try:
@@ -139,99 +142,103 @@ class PerMessageDeflateOffer(PerMessageCompressOffer, PerMessageDeflateMixin):
                     if val not in PerMessageDeflateMixin.WINDOW_SIZE_PERMISSIBLE_VALUES:
                         raise Exception("illegal extension parameter value '%s' for parameter '%s' of extension '%s'" % (val, p, cls.EXTENSION_NAME))
                     else:
-                        requestMaxWindowBits = val
+                        request_max_window_bits = val
 
             elif p == 'server_no_context_takeover':
                 # noinspection PySimplifyBooleanCheck
                 if val is not True:
                     raise Exception("illegal extension parameter value '%s' for parameter '%s' of extension '%s'" % (val, p, cls.EXTENSION_NAME))
                 else:
-                    requestNoContextTakeover = True
+                    request_no_context_takeover = True
 
             else:
                 raise Exception("illegal extension parameter '%s' for extension '%s'" % (p, cls.EXTENSION_NAME))
 
-        offer = cls(acceptNoContextTakeover,
-                    acceptMaxWindowBits,
-                    requestNoContextTakeover,
-                    requestMaxWindowBits)
+        offer = cls(accept_no_context_takeover,
+                    accept_max_window_bits,
+                    request_no_context_takeover,
+                    request_max_window_bits)
         return offer
 
     def __init__(self,
-                 acceptNoContextTakeover=True,
-                 acceptMaxWindowBits=True,
-                 requestNoContextTakeover=False,
-                 requestMaxWindowBits=0):
+                 accept_no_context_takeover=True,
+                 accept_max_window_bits=True,
+                 request_no_context_takeover=False,
+                 request_max_window_bits=0):
         """
-        Constructor.
 
-        :param acceptNoContextTakeover: Iff true, client accepts "no context takeover" feature.
-        :type acceptNoContextTakeover: bool
-        :param acceptMaxWindowBits: Iff true, client accepts setting "max window size".
-        :type acceptMaxWindowBits: bool
-        :param requestNoContextTakeover: Iff true, client request "no context takeover" feature.
-        :type requestNoContextTakeover: bool
-        :param requestMaxWindowBits: Iff non-zero, client requests given "max window size" - must be 8-15.
-        :type requestMaxWindowBits: int
+        :param accept_no_context_takeover: When ``True``, the client accepts the "no context takeover" feature.
+        :type accept_no_context_takeover: bool
+        :param accept_max_window_bits: When ``True``, the client accepts setting "max window size".
+        :type accept_max_window_bits: bool
+        :param request_no_context_takeover: When ``True``, the client request the "no context takeover" feature.
+        :type request_no_context_takeover: bool
+        :param request_max_window_bits: When non-zero, the client requests the given "max window size" (must be
+            and integer from the interval ``[8..15]``).
+        :type request_max_window_bits: int
         """
-        if type(acceptNoContextTakeover) != bool:
-            raise Exception("invalid type %s for acceptNoContextTakeover" % type(acceptNoContextTakeover))
+        if type(accept_no_context_takeover) != bool:
+            raise Exception("invalid type %s for accept_no_context_takeover" % type(accept_no_context_takeover))
 
-        self.acceptNoContextTakeover = acceptNoContextTakeover
+        self.accept_no_context_takeover = accept_no_context_takeover
 
-        if type(acceptMaxWindowBits) != bool:
-            raise Exception("invalid type %s for acceptMaxWindowBits" % type(acceptMaxWindowBits))
+        if type(accept_max_window_bits) != bool:
+            raise Exception("invalid type %s for accept_max_window_bits" % type(accept_max_window_bits))
 
-        self.acceptMaxWindowBits = acceptMaxWindowBits
+        self.accept_max_window_bits = accept_max_window_bits
 
-        if type(requestNoContextTakeover) != bool:
-            raise Exception("invalid type %s for requestNoContextTakeover" % type(requestNoContextTakeover))
+        if type(request_no_context_takeover) != bool:
+            raise Exception("invalid type %s for request_no_context_takeover" % type(request_no_context_takeover))
 
-        self.requestNoContextTakeover = requestNoContextTakeover
+        self.request_no_context_takeover = request_no_context_takeover
 
-        if requestMaxWindowBits != 0 and requestMaxWindowBits not in self.WINDOW_SIZE_PERMISSIBLE_VALUES:
-            raise Exception("invalid value %s for requestMaxWindowBits - permissible values %s" % (requestMaxWindowBits, self.WINDOW_SIZE_PERMISSIBLE_VALUES))
+        if request_max_window_bits != 0 and request_max_window_bits not in self.WINDOW_SIZE_PERMISSIBLE_VALUES:
+            raise Exception("invalid value %s for request_max_window_bits - permissible values %s" % (request_max_window_bits, self.WINDOW_SIZE_PERMISSIBLE_VALUES))
 
-        self.requestMaxWindowBits = requestMaxWindowBits
+        self.request_max_window_bits = request_max_window_bits
 
-    def getExtensionString(self):
+    def get_extension_string(self):
         """
         Returns the WebSocket extension configuration string as sent to the server.
 
-        :returns: str -- PMCE configuration string.
+        :returns: PMCE configuration string.
+        :rtype: str
         """
-        pmceString = self.EXTENSION_NAME
-        if self.acceptNoContextTakeover:
-            pmceString += "; client_no_context_takeover"
-        if self.acceptMaxWindowBits:
-            pmceString += "; client_max_window_bits"
-        if self.requestNoContextTakeover:
-            pmceString += "; server_no_context_takeover"
-        if self.requestMaxWindowBits != 0:
-            pmceString += "; server_max_window_bits=%d" % self.requestMaxWindowBits
-        return pmceString
+        pmce_string = self.EXTENSION_NAME
+        if self.accept_no_context_takeover:
+            pmce_string += "; client_no_context_takeover"
+        if self.accept_max_window_bits:
+            pmce_string += "; client_max_window_bits"
+        if self.request_no_context_takeover:
+            pmce_string += "; server_no_context_takeover"
+        if self.request_max_window_bits != 0:
+            pmce_string += "; server_max_window_bits=%d" % self.request_max_window_bits
+        return pmce_string
 
     def __json__(self):
         """
         Returns a JSON serializable object representation.
 
-        :returns: object -- JSON serializable representation.
+        :returns: JSON serializable representation.
+        :rtype: dict
         """
         return {'extension': self.EXTENSION_NAME,
-                'acceptNoContextTakeover': self.acceptNoContextTakeover,
-                'acceptMaxWindowBits': self.acceptMaxWindowBits,
-                'requestNoContextTakeover': self.requestNoContextTakeover,
-                'requestMaxWindowBits': self.requestMaxWindowBits}
+                'accept_no_context_takeover': self.accept_no_context_takeover,
+                'accept_max_window_bits': self.accept_max_window_bits,
+                'request_no_context_takeover': self.request_no_context_takeover,
+                'request_max_window_bits': self.request_max_window_bits}
 
     def __repr__(self):
         """
         Returns Python object representation that can be eval'ed to reconstruct the object.
 
-        :returns: str -- Python string representation.
+        :returns: Python string representation.
+        :rtype: str
         """
-        return "PerMessageDeflateOffer(acceptNoContextTakeover = %s, acceptMaxWindowBits = %s, requestNoContextTakeover = %s, requestMaxWindowBits = %s)" % (self.acceptNoContextTakeover, self.acceptMaxWindowBits, self.requestNoContextTakeover, self.requestMaxWindowBits)
+        return "PerMessageDeflateOffer(accept_no_context_takeover = %s, accept_max_window_bits = %s, request_no_context_takeover = %s, request_max_window_bits = %s)" % (self.accept_no_context_takeover, self.accept_max_window_bits, self.request_no_context_takeover, self.request_max_window_bits)
 
 
+@public
 class PerMessageDeflateOfferAccept(PerMessageCompressOfferAccept, PerMessageDeflateMixin):
     """
     Set of parameters with which to accept an `permessage-deflate` offer
@@ -240,112 +247,123 @@ class PerMessageDeflateOfferAccept(PerMessageCompressOfferAccept, PerMessageDefl
 
     def __init__(self,
                  offer,
-                 requestNoContextTakeover=False,
-                 requestMaxWindowBits=0,
-                 noContextTakeover=None,
-                 windowBits=None,
-                 memLevel=None):
+                 request_no_context_takeover=False,
+                 request_max_window_bits=0,
+                 no_context_takeover=None,
+                 window_bits=None,
+                 mem_level=None,
+                 max_message_size=None):
         """
-        Constructor.
 
         :param offer: The offer being accepted.
         :type offer: Instance of :class:`autobahn.compress.PerMessageDeflateOffer`.
-        :param requestNoContextTakeover: Iff true, server request "no context takeover" feature.
-        :type requestNoContextTakeover: bool
-        :param requestMaxCompressLevel: Iff non-zero, server requests given "maximum compression level" - must be 1-9.
-        :type requestMaxCompressLevel: int
-        :param noContextTakeover: Override server ("server-to-client direction") context takeover (this must be compatible with offer).
-        :type noContextTakeover: bool
-        :param windowBits: Override server ("server-to-client direction") window size (this must be compatible with offer).
-        :type windowBits: int
-        :param memLevel: Set server ("server-to-client direction") memory level.
-        :type memLevel: int
+        :param request_no_context_takeover: When ``True``, the server requests the "no context takeover" feature.
+        :type request_no_context_takeover: bool
+        :param request_max_window_bits: When non-zero, the server requests the given "max window size" (must be
+            and integer from the interval ``[8..15]``).
+        :param request_max_window_bits: int
+        :param no_context_takeover: Override server ("server-to-client direction") context takeover (this must
+                be compatible with the offer).
+        :type no_context_takeover: bool
+        :param window_bits: Override server ("server-to-client direction") window size (this must be
+                compatible with the offer).
+        :type window_bits: int
+        :param mem_level: Set server ("server-to-client direction") memory level.
+        :type mem_level: int
         """
         if not isinstance(offer, PerMessageDeflateOffer):
             raise Exception("invalid type %s for offer" % type(offer))
 
         self.offer = offer
 
-        if type(requestNoContextTakeover) != bool:
-            raise Exception("invalid type %s for requestNoContextTakeover" % type(requestNoContextTakeover))
+        if type(request_no_context_takeover) != bool:
+            raise Exception("invalid type %s for request_no_context_takeover" % type(request_no_context_takeover))
 
-        if requestNoContextTakeover and not offer.acceptNoContextTakeover:
-            raise Exception("invalid value %s for requestNoContextTakeover - feature unsupported by client" % requestNoContextTakeover)
+        if request_no_context_takeover and not offer.accept_no_context_takeover:
+            raise Exception("invalid value %s for request_no_context_takeover - feature unsupported by client" % request_no_context_takeover)
 
-        self.requestNoContextTakeover = requestNoContextTakeover
+        self.request_no_context_takeover = request_no_context_takeover
 
-        if requestMaxWindowBits != 0 and requestMaxWindowBits not in self.WINDOW_SIZE_PERMISSIBLE_VALUES:
-            raise Exception("invalid value %s for requestMaxWindowBits - permissible values %s" % (requestMaxWindowBits, self.WINDOW_SIZE_PERMISSIBLE_VALUES))
+        if request_max_window_bits != 0 and request_max_window_bits not in self.WINDOW_SIZE_PERMISSIBLE_VALUES:
+            raise Exception("invalid value %s for request_max_window_bits - permissible values %s" % (request_max_window_bits, self.WINDOW_SIZE_PERMISSIBLE_VALUES))
 
-        if requestMaxWindowBits != 0 and not offer.acceptMaxWindowBits:
-            raise Exception("invalid value %s for requestMaxWindowBits - feature unsupported by client" % requestMaxWindowBits)
+        if request_max_window_bits != 0 and not offer.accept_max_window_bits:
+            raise Exception("invalid value %s for request_max_window_bits - feature unsupported by client" % request_max_window_bits)
 
-        self.requestMaxWindowBits = requestMaxWindowBits
+        self.request_max_window_bits = request_max_window_bits
 
-        if noContextTakeover is not None:
-            if type(noContextTakeover) != bool:
-                raise Exception("invalid type %s for noContextTakeover" % type(noContextTakeover))
+        if no_context_takeover is not None:
+            if type(no_context_takeover) != bool:
+                raise Exception("invalid type %s for no_context_takeover" % type(no_context_takeover))
 
-            if offer.requestNoContextTakeover and not noContextTakeover:
-                raise Exception("invalid value %s for noContextTakeover - client requested feature" % noContextTakeover)
+            if offer.request_no_context_takeover and not no_context_takeover:
+                raise Exception("invalid value %s for no_context_takeover - client requested feature" % no_context_takeover)
 
-        self.noContextTakeover = noContextTakeover
+        self.no_context_takeover = no_context_takeover
 
-        if windowBits is not None:
-            if windowBits not in self.WINDOW_SIZE_PERMISSIBLE_VALUES:
-                raise Exception("invalid value %s for windowBits - permissible values %s" % (windowBits, self.WINDOW_SIZE_PERMISSIBLE_VALUES))
+        if window_bits is not None:
+            if window_bits not in self.WINDOW_SIZE_PERMISSIBLE_VALUES:
+                raise Exception("invalid value %s for window_bits - permissible values %s" % (window_bits, self.WINDOW_SIZE_PERMISSIBLE_VALUES))
 
-            if offer.requestMaxWindowBits != 0 and windowBits > offer.requestMaxWindowBits:
-                raise Exception("invalid value %s for windowBits - client requested lower maximum value" % windowBits)
+            if offer.request_max_window_bits != 0 and window_bits > offer.request_max_window_bits:
+                raise Exception("invalid value %s for window_bits - client requested lower maximum value" % window_bits)
 
-        self.windowBits = windowBits
+        self.window_bits = window_bits
 
-        if memLevel is not None:
-            if memLevel not in self.MEM_LEVEL_PERMISSIBLE_VALUES:
-                raise Exception("invalid value %s for memLevel - permissible values %s" % (memLevel, self.MEM_LEVEL_PERMISSIBLE_VALUES))
+        if mem_level is not None:
+            if mem_level not in self.MEM_LEVEL_PERMISSIBLE_VALUES:
+                raise Exception("invalid value %s for mem_level - permissible values %s" % (mem_level, self.MEM_LEVEL_PERMISSIBLE_VALUES))
 
-        self.memLevel = memLevel
+        self.mem_level = mem_level
+        self.max_message_size = max_message_size  # clamp/check values..?
 
-    def getExtensionString(self):
+    def get_extension_string(self):
         """
         Returns the WebSocket extension configuration string as sent to the server.
 
-        :returns: str -- PMCE configuration string.
+        :returns: PMCE configuration string.
+        :rtype: str
         """
-        pmceString = self.EXTENSION_NAME
-        if self.offer.requestNoContextTakeover:
-            pmceString += "; server_no_context_takeover"
-        if self.offer.requestMaxWindowBits != 0:
-            pmceString += "; server_max_window_bits=%d" % self.offer.requestMaxWindowBits
-        if self.requestNoContextTakeover:
-            pmceString += "; client_no_context_takeover"
-        if self.requestMaxWindowBits != 0:
-            pmceString += "; client_max_window_bits=%d" % self.requestMaxWindowBits
-        return pmceString
+        pmce_string = self.EXTENSION_NAME
+        if self.offer.request_no_context_takeover:
+            pmce_string += "; server_no_context_takeover"
+        if self.offer.request_max_window_bits != 0:
+            pmce_string += "; server_max_window_bits=%d" % self.offer.request_max_window_bits
+        if self.request_no_context_takeover:
+            pmce_string += "; client_no_context_takeover"
+        if self.request_max_window_bits != 0:
+            pmce_string += "; client_max_window_bits=%d" % self.request_max_window_bits
+        return pmce_string
 
     def __json__(self):
         """
         Returns a JSON serializable object representation.
 
-        :returns: object -- JSON serializable representation.
+        :returns: JSON serializable representation.
+        :rtype: dict
         """
-        return {'extension': self.EXTENSION_NAME,
-                'offer': self.offer.__json__(),
-                'requestNoContextTakeover': self.requestNoContextTakeover,
-                'requestMaxWindowBits': self.requestMaxWindowBits,
-                'noContextTakeover': self.noContextTakeover,
-                'windowBits': self.windowBits,
-                'memLevel': self.memLevel}
+        return {
+            'extension': self.EXTENSION_NAME,
+            'offer': self.offer.__json__(),
+            'request_no_context_takeover': self.request_no_context_takeover,
+            'request_max_window_bits': self.request_max_window_bits,
+            'no_context_takeover': self.no_context_takeover,
+            'window_bits': self.window_bits,
+            'mem_level': self.mem_level,
+            'max_message_size': self.max_message_size,
+        }
 
     def __repr__(self):
         """
         Returns Python object representation that can be eval'ed to reconstruct the object.
 
-        :returns: str -- Python string representation.
+        :returns: Python string representation.
+        :rtype: str
         """
-        return "PerMessageDeflateOfferAccept(offer = %s, requestNoContextTakeover = %s, requestMaxWindowBits = %s, noContextTakeover = %s, windowBits = %s, memLevel = %s)" % (self.offer.__repr__(), self.requestNoContextTakeover, self.requestMaxWindowBits, self.noContextTakeover, self.windowBits, self.memLevel)
+        return "PerMessageDeflateOfferAccept(offer = %s, request_no_context_takeover = %s, request_max_window_bits = %s, no_context_takeover = %s, window_bits = %s, mem_level = %s, max_message_size = %s)" % (self.offer.__repr__(), self.request_no_context_takeover, self.request_max_window_bits, self.no_context_takeover, self.window_bits, self.mem_level, self.max_message_size)
 
 
+@public
 class PerMessageDeflateResponse(PerMessageCompressResponse, PerMessageDeflateMixin):
     """
     Set of parameters for `permessage-deflate` responded by server.
@@ -359,7 +377,8 @@ class PerMessageDeflateResponse(PerMessageCompressResponse, PerMessageDeflateMix
         :param params: Output from :func:`autobahn.websocket.WebSocketProtocol._parseExtensionsHeader`.
         :type params: list
 
-        :returns: object -- A new instance of :class:`autobahn.compress.PerMessageDeflateResponse`.
+        :returns: A new instance of :class:`autobahn.compress.PerMessageDeflateResponse`.
+        :rtype: obj
         """
         client_max_window_bits = 0
         client_no_context_takeover = False
@@ -423,6 +442,17 @@ class PerMessageDeflateResponse(PerMessageCompressResponse, PerMessageDeflateMix
                  client_no_context_takeover,
                  server_max_window_bits,
                  server_no_context_takeover):
+        """
+
+        :param client_max_window_bits: FIXME
+        :type client_max_window_bits: int
+        :param client_no_context_takeover: FIXME
+        :type client_no_context_takeover: bool
+        :param server_max_window_bits: FIXME
+        :type server_max_window_bits: int
+        :param server_no_context_takeover: FIXME
+        :type server_no_context_takeover: bool
+        """
         self.client_max_window_bits = client_max_window_bits
         self.client_no_context_takeover = client_no_context_takeover
         self.server_max_window_bits = server_max_window_bits
@@ -432,7 +462,8 @@ class PerMessageDeflateResponse(PerMessageCompressResponse, PerMessageDeflateMix
         """
         Returns a JSON serializable object representation.
 
-        :returns: object -- JSON serializable representation.
+        :returns: JSON serializable representation.
+        :rtype: dict
         """
         return {'extension': self.EXTENSION_NAME,
                 'client_max_window_bits': self.client_max_window_bits,
@@ -444,11 +475,13 @@ class PerMessageDeflateResponse(PerMessageCompressResponse, PerMessageDeflateMix
         """
         Returns Python object representation that can be eval'ed to reconstruct the object.
 
-        :returns: str -- Python string representation.
+        :returns: Python string representation.
+        :rtype: str
         """
         return "PerMessageDeflateResponse(client_max_window_bits = %s, client_no_context_takeover = %s, server_max_window_bits = %s, server_no_context_takeover = %s)" % (self.client_max_window_bits, self.client_no_context_takeover, self.server_max_window_bits, self.server_no_context_takeover)
 
 
+@public
 class PerMessageDeflateResponseAccept(PerMessageCompressResponseAccept, PerMessageDeflateMixin):
     """
     Set of parameters with which to accept an `permessage-deflate` response
@@ -457,69 +490,72 @@ class PerMessageDeflateResponseAccept(PerMessageCompressResponseAccept, PerMessa
 
     def __init__(self,
                  response,
-                 noContextTakeover=None,
-                 windowBits=None,
-                 memLevel=None):
+                 no_context_takeover=None,
+                 window_bits=None,
+                 mem_level=None,
+                 max_message_size=None):
         """
-        Constructor.
 
         :param response: The response being accepted.
         :type response: Instance of :class:`autobahn.compress.PerMessageDeflateResponse`.
-        :param noContextTakeover: Override client ("client-to-server direction") context takeover (this must be compatible with response).
-        :type noContextTakeover: bool
-        :param windowBits: Override client ("client-to-server direction") window size (this must be compatible with response).
-        :type windowBits: int
-        :param memLevel: Set client ("client-to-server direction") memory level.
-        :type memLevel: int
+        :param no_context_takeover: Override client ("client-to-server direction") context takeover (this must be compatible with response).
+        :type no_context_takeover: bool
+        :param window_bits: Override client ("client-to-server direction") window size (this must be compatible with response).
+        :type window_bits: int
+        :param mem_level: Set client ("client-to-server direction") memory level.
+        :type mem_level: int
         """
         if not isinstance(response, PerMessageDeflateResponse):
             raise Exception("invalid type %s for response" % type(response))
 
         self.response = response
 
-        if noContextTakeover is not None:
-            if type(noContextTakeover) != bool:
-                raise Exception("invalid type %s for noContextTakeover" % type(noContextTakeover))
+        if no_context_takeover is not None:
+            if type(no_context_takeover) != bool:
+                raise Exception("invalid type %s for no_context_takeover" % type(no_context_takeover))
 
-            if response.client_no_context_takeover and not noContextTakeover:
-                raise Exception("invalid value %s for noContextTakeover - server requested feature" % noContextTakeover)
+            if response.client_no_context_takeover and not no_context_takeover:
+                raise Exception("invalid value %s for no_context_takeover - server requested feature" % no_context_takeover)
 
-        self.noContextTakeover = noContextTakeover
+        self.no_context_takeover = no_context_takeover
 
-        if windowBits is not None:
-            if windowBits not in self.WINDOW_SIZE_PERMISSIBLE_VALUES:
-                raise Exception("invalid value %s for windowBits - permissible values %s" % (windowBits, self.WINDOW_SIZE_PERMISSIBLE_VALUES))
+        if window_bits is not None:
+            if window_bits not in self.WINDOW_SIZE_PERMISSIBLE_VALUES:
+                raise Exception("invalid value %s for window_bits - permissible values %s" % (window_bits, self.WINDOW_SIZE_PERMISSIBLE_VALUES))
 
-            if response.client_max_window_bits != 0 and windowBits > response.client_max_window_bits:
-                raise Exception("invalid value %s for windowBits - server requested lower maximum value" % windowBits)
+            if response.client_max_window_bits != 0 and window_bits > response.client_max_window_bits:
+                raise Exception("invalid value %s for window_bits - server requested lower maximum value" % window_bits)
 
-        self.windowBits = windowBits
+        self.window_bits = window_bits
 
-        if memLevel is not None:
-            if memLevel not in self.MEM_LEVEL_PERMISSIBLE_VALUES:
-                raise Exception("invalid value %s for memLevel - permissible values %s" % (memLevel, self.MEM_LEVEL_PERMISSIBLE_VALUES))
+        if mem_level is not None:
+            if mem_level not in self.MEM_LEVEL_PERMISSIBLE_VALUES:
+                raise Exception("invalid value %s for mem_level - permissible values %s" % (mem_level, self.MEM_LEVEL_PERMISSIBLE_VALUES))
 
-        self.memLevel = memLevel
+        self.mem_level = mem_level
+        self.max_message_size = max_message_size
 
     def __json__(self):
         """
         Returns a JSON serializable object representation.
 
-        :returns: object -- JSON serializable representation.
+        :returns: JSON serializable representation.
+        :rtype: dict
         """
         return {'extension': self.EXTENSION_NAME,
                 'response': self.response.__json__(),
-                'noContextTakeover': self.noContextTakeover,
-                'windowBits': self.windowBits,
-                'memLevel': self.memLevel}
+                'no_context_takeover': self.no_context_takeover,
+                'window_bits': self.window_bits,
+                'mem_level': self.mem_level}
 
     def __repr__(self):
         """
         Returns Python object representation that can be eval'ed to reconstruct the object.
 
-        :returns: str -- Python string representation.
+        :returns: Python string representation.
+        :rtype: str
         """
-        return "PerMessageDeflateResponseAccept(response = %s, noContextTakeover = %s, windowBits = %s, memLevel = %s)" % (self.response.__repr__(), self.noContextTakeover, self.windowBits, self.memLevel)
+        return "PerMessageDeflateResponseAccept(response = %s, no_context_takeover = %s, window_bits = %s, mem_level = %s)" % (self.response.__repr__(), self.no_context_takeover, self.window_bits, self.mem_level)
 
 
 # noinspection PyArgumentList
@@ -531,35 +567,40 @@ class PerMessageDeflate(PerMessageCompress, PerMessageDeflateMixin):
     DEFAULT_MEM_LEVEL = 8
 
     @classmethod
-    def createFromResponseAccept(cls, isServer, accept):
+    def create_from_response_accept(cls, is_server, accept):
         # accept: instance of PerMessageDeflateResponseAccept
-        pmce = cls(isServer,
-                   accept.response.server_no_context_takeover,
-                   accept.noContextTakeover if accept.noContextTakeover is not None else accept.response.client_no_context_takeover,
-                   accept.response.server_max_window_bits,
-                   accept.windowBits if accept.windowBits is not None else accept.response.client_max_window_bits,
-                   accept.memLevel)
+        pmce = cls(
+            is_server,
+            accept.response.server_no_context_takeover,
+            accept.no_context_takeover if accept.no_context_takeover is not None else accept.response.client_no_context_takeover,
+            accept.response.server_max_window_bits,
+            accept.window_bits if accept.window_bits is not None else accept.response.client_max_window_bits,
+            accept.mem_level,
+            accept.max_message_size,
+        )
         return pmce
 
     @classmethod
-    def createFromOfferAccept(cls, isServer, accept):
+    def create_from_offer_accept(cls, is_server, accept):
         # accept: instance of PerMessageDeflateOfferAccept
-        pmce = cls(isServer,
-                   accept.noContextTakeover if accept.noContextTakeover is not None else accept.offer.requestNoContextTakeover,
-                   accept.requestNoContextTakeover,
-                   accept.windowBits if accept.windowBits is not None else accept.offer.requestMaxWindowBits,
-                   accept.requestMaxWindowBits,
-                   accept.memLevel)
+        pmce = cls(is_server,
+                   accept.no_context_takeover if accept.no_context_takeover is not None else accept.offer.request_no_context_takeover,
+                   accept.request_no_context_takeover,
+                   accept.window_bits if accept.window_bits is not None else accept.offer.request_max_window_bits,
+                   accept.request_max_window_bits,
+                   accept.mem_level,
+                   accept.max_message_size,)
         return pmce
 
     def __init__(self,
-                 isServer,
+                 is_server,
                  server_no_context_takeover,
                  client_no_context_takeover,
                  server_max_window_bits,
                  client_max_window_bits,
-                 mem_level):
-        self._isServer = isServer
+                 mem_level,
+                 max_message_size=None):
+        self._is_server = is_server
 
         self.server_no_context_takeover = server_no_context_takeover
         self.client_no_context_takeover = client_no_context_takeover
@@ -568,13 +609,14 @@ class PerMessageDeflate(PerMessageCompress, PerMessageDeflateMixin):
         self.client_max_window_bits = client_max_window_bits if client_max_window_bits != 0 else self.DEFAULT_WINDOW_BITS
 
         self.mem_level = mem_level if mem_level else self.DEFAULT_MEM_LEVEL
+        self.max_message_size = max_message_size  # None means "no limit"
 
         self._compressor = None
         self._decompressor = None
 
     def __json__(self):
         return {'extension': self.EXTENSION_NAME,
-                'isServer': self._isServer,
+                'is_server': self._is_server,
                 'server_no_context_takeover': self.server_no_context_takeover,
                 'client_no_context_takeover': self.client_no_context_takeover,
                 'server_max_window_bits': self.server_max_window_bits,
@@ -582,38 +624,40 @@ class PerMessageDeflate(PerMessageCompress, PerMessageDeflateMixin):
                 'mem_level': self.mem_level}
 
     def __repr__(self):
-        return "PerMessageDeflate(isServer = %s, server_no_context_takeover = %s, client_no_context_takeover = %s, server_max_window_bits = %s, client_max_window_bits = %s, mem_level = %s)" % (self._isServer, self.server_no_context_takeover, self.client_no_context_takeover, self.server_max_window_bits, self.client_max_window_bits, self.mem_level)
+        return "PerMessageDeflate(is_server = %s, server_no_context_takeover = %s, client_no_context_takeover = %s, server_max_window_bits = %s, client_max_window_bits = %s, mem_level = %s)" % (self._is_server, self.server_no_context_takeover, self.client_no_context_takeover, self.server_max_window_bits, self.client_max_window_bits, self.mem_level)
 
-    def startCompressMessage(self):
-        # compressobj([level[, method[, wbits[, memlevel[, strategy]]]]])
+    def start_compress_message(self):
+        # compressobj([level[, method[, wbits[, mem_level[, strategy]]]]])
         # http://bugs.python.org/issue19278
         # http://hg.python.org/cpython/rev/c54c8e71b79a
-        if self._isServer:
+        if self._is_server:
             if self._compressor is None or self.server_no_context_takeover:
                 self._compressor = zlib.compressobj(zlib.Z_DEFAULT_COMPRESSION, zlib.DEFLATED, -self.server_max_window_bits, self.mem_level)
         else:
             if self._compressor is None or self.client_no_context_takeover:
                 self._compressor = zlib.compressobj(zlib.Z_DEFAULT_COMPRESSION, zlib.DEFLATED, -self.client_max_window_bits, self.mem_level)
 
-    def compressMessageData(self, data):
+    def compress_message_data(self, data):
         return self._compressor.compress(data)
 
-    def endCompressMessage(self):
+    def end_compress_message(self):
         data = self._compressor.flush(zlib.Z_SYNC_FLUSH)
         return data[:-4]
 
-    def startDecompressMessage(self):
-        if self._isServer:
+    def start_decompress_message(self):
+        if self._is_server:
             if self._decompressor is None or self.client_no_context_takeover:
                 self._decompressor = zlib.decompressobj(-self.client_max_window_bits)
         else:
             if self._decompressor is None or self.server_no_context_takeover:
                 self._decompressor = zlib.decompressobj(-self.server_max_window_bits)
 
-    def decompressMessageData(self, data):
+    def decompress_message_data(self, data):
+        if self.max_message_size is not None:
+            return self._decompressor.decompress(data, self.max_message_size)
         return self._decompressor.decompress(data)
 
-    def endDecompressMessage(self):
+    def end_decompress_message(self):
         # Eat stripped LEN and NLEN field of a non-compressed block added
         # for Z_SYNC_FLUSH.
         self._decompressor.decompress(b'\x00\x00\xff\xff')
